import oracledb from 'oracledb'
import {getPoolById} from './db.js'
import {queries} from './Queries.js'
import { log } from '../Controllers/LogController.js'

let sharedConnection = {}

/**
     * execute Oracle query
     * 
     * @param {String} sql query key to be executed on oracle database
     * @param {Object} binds query params keys valued
     * @param {Object} options fetch size, autoCommit, outFormat, etc
     * 
     * Se autoCommit for false, connectionKey é obrigatório
*/
async function executeQuery({ sql, binds = {}, nrorg, rawReturn = false, newCode = false, type='query', autoCommit = true, connectionKey }) {
    try {

        let pool
        let connection
        
        if (!autoCommit && !sharedConnection[connectionKey]) {
            pool = await getPoolById(nrorg)
            connection = await pool.getConnection()
            sharedConnection[connectionKey] = connection
        } else if (!autoCommit && sharedConnection[connectionKey]) {
            connection = sharedConnection[connectionKey]
        } else {
            pool = await getPoolById(nrorg)
            connection = await pool.getConnection()
        }

        if (newCode) {
            type = 'procedure'
            sql = 'NOVO_CODIGO'
            binds = {
                ...binds,
                output: { type: oracledb.STRING, dir: oracledb.BIND_OUT }
            }
        }

        let result;

        Object.keys(binds).forEach(key => {
            const value = binds[key]
            if (typeof binds[key] === 'number') {
                binds[key] = { val: value, type: oracledb.DB_TYPE_BINARY_FLOAT }
            } else if (value instanceof Date){
                binds[key] = { val: value, type: oracledb.DB_TYPE_DATE }
            } else {
                binds[key] = { val: value, type: oracledb.STRING }
            }
            binds[key].dir = oracledb.BIND_IN
        })

        if (type === 'query') {
            // Configuração adicionada para ler o campo CORESJSON da tabela NOCASHPARAMS como String 
            oracledb.fetchAsString = [oracledb.CLOB]
            
            result = await connection.execute(queries[sql], binds, {
                outFormat: oracledb.OUT_FORMAT_OBJECT, // Output format as objects
                extendedMetaData: true, // Retrieve column names in the result
                autoCommit: autoCommit, // Commit after each statement
            })
        } else if (type === 'procedure') {
            result = await connection.execute(
                queries[sql], 
                {
                    ...binds,
                    output: { type: oracledb.STRING, dir: oracledb.BIND_OUT }
                }, 
                {
                    outFormat: oracledb.OUT_FORMAT_OBJECT, // Output format as objects
                    extendedMetaData: true, // Retrieve column names in the result
                    autoCommit: true // Commit after each statement
                })
        }

        if (autoCommit) await connection.close()
        
        if (result.outBinds && result.outBinds.output) {
            return result.outBinds.output;
        }

        if (result.rows && result.rows.length === 1 && !rawReturn) {
            return result.rows[0]
        }
        return result.rows || result
    } catch (err) {
        log({ type: 'error', message: `Error executing query: ${sql}\n${err.message}` })
        throw err
    }
}

async function commit ({ nrorg, connectionKey }) {
    try {
        if (sharedConnection[connectionKey]) {
            await sharedConnection[connectionKey].commit()
            await sharedConnection[connectionKey].close()
            delete sharedConnection[connectionKey]
        }
    } catch (err) {
        log({ type: 'error', message: `Error committing transaction:\n${err.message}` })
    }
}

async function rollback ({ nrorg, connectionKey }) {
    try {
        if (sharedConnection[connectionKey]) {
            await sharedConnection[connectionKey].rollback()
            await sharedConnection[connectionKey].close()
            delete sharedConnection[connectionKey]
        }
    } catch (err) {
        log({ type: 'error', message: `Error rolling back transaction:\n${err.message}` })
    }
}

export { executeQuery, commit, rollback }