import { paymentApiRequest } from './../Requests/RequestModule.js'
import { log } from './LogController.js'
import { sanitizeNumber, valueInCents } from './../Libs/Util.js'
import { getRetailGeneralParamsBackend } from './RetailParamsController.js'

const doPaymentThroughAPI = async ({ totalValue, flag, cardExpiration, cardNumber, cardToken, cardCVV, cardHolder, mode, NRTRANSACAO }) => {    
    let uri = `payment/${mode}`
    let payload = {}
    let response = null

    if (mode === 'credit') {
        payload = await buildCardPayload(totalValue, flag, cardExpiration, cardNumber, cardToken, cardCVV, cardHolder, NRTRANSACAO)
    } else if (mode === 'pix') {
        payload = await buildPixPayload(totalValue, NRTRANSACAO)
    }

    try {
        response = await paymentApiRequest(uri, payload)
    } catch (error) {
        log({ type: 'error', message: `Ocorreu um erro ao tentar realizar o pagamento pela API: ${error.message}` })
        // console.log(error)

        if (error?.response?.data?.message) {
            log({ type: 'error', message: error.response.data.message })
        }
    }

    return response
}

const buildCardPayload = async (totalValue, flag, cardExpiration, cardNumber, cardToken, cardCVV, cardHolder, NRTRANSACAO) => {
    const retailParams = await getRetailGeneralParamsBackend()
    const merchantId = retailParams?.MERCHANTID ? retailParams.MERCHANTID : ''
    const merchantKey = retailParams?.MERCHANTKEY ? retailParams.MERCHANTKEY : ''
    const paymentCompany = retailParams?.PAYMENTCOMPANY ? retailParams.PAYMENTCOMPANY : ''

    const payload = {
        paymentCompany,
        orderId: NRTRANSACAO,
        installments: getInstallments(),
        installmentType: getInstallmentType(),
        authorizer: flag,
        postponeConfirmation: true, // confirmação tardia
        amount: valueInCents(totalValue),
        card: {
            holder: cardHolder, 
            expiryDate: sanitizeNumber(cardExpiration),
            securityCode: sanitizeNumber(cardCVV)
        },
        credentials: {
            merchantId,
            merchantKey
        }
    }

    if (cardNumber) {
        payload.card.number = sanitizeNumber(cardNumber)
    } else {
        payload.card.token = cardToken
    }

    return payload
}

const buildPixPayload = async (totalValue, NRTRANSACAO) => {
    const retailParams = await getRetailGeneralParamsBackend()
    const merchantId = retailParams?.MERCHANTID ? retailParams.MERCHANTID : ''
    const merchantKey = retailParams?.MERCHANTKEY ? retailParams.MERCHANTKEY : ''
    const paymentCompany = retailParams?.PAYMENTCOMPANY ? retailParams.PAYMENTCOMPANY : ''

    return {
        paymentCompany,
        orderId: NRTRANSACAO,
        installments: getInstallments(),
        installmentType: getInstallmentType(),
        amount: valueInCents(totalValue),
        credentials: {
            merchantId,
            merchantKey
        }    
    }
}

const getInstallments = () => {
    const HARDCODED_INSTALLMENTS = '1'
    return HARDCODED_INSTALLMENTS
}

const getInstallmentType = () => {
    const HARDCODED_INSTALLMENT_TYPE = 'V'
    return HARDCODED_INSTALLMENT_TYPE
}

const confirmPayment = async (nit) => {    
    let uri = `payment/confirm/${nit}`
    let payload = {}
    let response = null

    payload = await buildSimplifiedPayload()

    try {
        response = await paymentApiRequest(uri, payload)
    } catch (error) {
        log({ type: 'error', message: `Ocorreu um erro ao tentar confirmar o pagamento pela API: ${error.message}` })

        if (error?.response?.data?.message) {
            log({ type: 'error', message: error.response.data.message })
        }
    }

    return response
}

const buildSimplifiedPayload = async () => {
    const retailParams = await getRetailGeneralParamsBackend()
    const merchantId = retailParams?.MERCHANTID ? retailParams.MERCHANTID : ''
    const merchantKey = retailParams?.MERCHANTKEY ? retailParams.MERCHANTKEY : ''
    const paymentCompany = retailParams?.PAYMENTCOMPANY ? retailParams.PAYMENTCOMPANY : ''

    return {
        paymentCompany,
        credentials: {
            merchantId,
            merchantKey
        }    
    }
}

const cancelPayment = async (nit) => {    
    let uri = `payment/cancel/${nit}`
    let payload = {}
    let response = null

    payload = await buildSimplifiedPayload()

    try {
        response = await paymentApiRequest(uri, payload)
    } catch (error) {
        log({ type: 'error', message: `Ocorreu um erro ao tentar cancelar o pagamento pela API: ${error.message}` })

        if (error?.response?.data?.message) {
            log({ type: 'error', message: error.response.data.message })
        }
    }

    return response
}

const consultPayment = async (nit) => {    
    let uri = `payment/consult/${nit}`
    let payload = {}
    let response = null

    payload = await buildSimplifiedPayload()

    try {
        response = await paymentApiRequest(uri, payload)
    } catch (error) {
        log({ type: 'error', message: `Ocorreu um erro ao tentar consultar o pagamento pela API: ${error.message}` })

        if (error?.response?.data?.message) {
            log({ type: 'error', message: error.response.data.message })
        }
    }

    return response
}

export { doPaymentThroughAPI, confirmPayment, cancelPayment, consultPayment}
